package airline.ejb;

/**
 * This class is part of Middlegen airlines, and it is
 * a CMP EJB accessing the persons table.
 *
 * @author <a href="http://boss.bekk.no/boss/middlegen/">Middlegen</a>
 *
 * @ejb:finder
 *    signature="java.util.Collection findByFirstNameAndLastName(java.lang.String firstName,java.lang.String lastName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.firstName = ?1 AND o.lastName = ?2"
 *
 * @ejb.bean
 *    type="CMP"
 *    cmp-version="2.x"
 *    name="Person"
 *    local-jndi-name="airline/PersonLocalHome"
 *    view-type="local"
 *    primkey-field="personId"
 *
 * @ejb.value-object
 *    name="Person"
 *    extends="airline.interfaces.PersonLightValue"
 *    match="*"
 *    instantiation="eager"
 *
 * @ejb.value-object
 *    name="PersonLight"
 *    match="light"
 *
 * @ejb.facade 
 *    type="Stateless"
 *    view-type="both"
 *
 *
 * @ejb.finder
 *    signature="java.util.Collection findAll()"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT OBJECT(o) FROM Person o"
 *    description="Find all entities."
 *
 * @ejb.finder
 *    signature="java.util.Collection findByFirstName(java.lang.String firstName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.firstName = ?1"
 *    description="Finder for not indexed column first_name."
 *
 * @ejb.finder
 *    signature="java.util.Collection findByLastName(java.lang.String lastName)"
 *    result-type-mapping="Local"
 *    method-intf="LocalHome"
 *    query="SELECT DISTINCT OBJECT(o) FROM Person o WHERE o.lastName = ?1"
 *    description="Finder for not indexed column last_name."
 *
 * @ejb.persistence table-name="persons"
 * @ejb.transaction type="Required" 
 *
 * @weblogic.data-source-name java:/DefaultDS
 *
 * @ejb.ejb-ref 
 *    ejb-name="SequenceSession" 
 *    ref-name="ejb/SequenceSession" 
 *    view-type="local" 
 * 
 */
public abstract class PersonBean 
    implements javax.ejb.EntityBean {

   /**
    * Returns the personId
    *
    * @return the personId
    *
    * @ejb.pk-field
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="person_id"
    * @ejb.value-object match="light"
    */
   public abstract java.lang.Integer getPersonId();

   /**
    * Sets the personId
    *
    * @param personId the new personId value
    */
   public abstract void setPersonId(java.lang.Integer personId);

   /**
    * Returns the firstName
    *
    * @return the firstName
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="first_name"
    * @ejb.value-object match="light"
    */
   public abstract java.lang.String getFirstName();

   /**
    * Sets the firstName
    *
    * @param firstName the new firstName value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setFirstName(java.lang.String firstName);

   /**
    * Returns the lastName
    *
    * @return the lastName
    *
    * @ejb.interface-method view-type="local"
    * @ejb.persistent-field
    * @ejb.persistence column-name="last_name"
    * @ejb.value-object match="light"
    */
   public abstract java.lang.String getLastName();

   /**
    * Sets the lastName
    *
    * @param lastName the new lastName value
    * @ejb.interface-method view-type="local"
    */
   public abstract void setLastName(java.lang.String lastName);

   /**
    * This is a bi-directional one-to-many relationship CMR method
    *
    * @return a airline.interfaces.ReservationLocal of related airline.interfaces.ReservationLocal.
    *
    * @ejb.interface-method view-type="local"
    *
    * @ejb.relation
    *    name="persons-cmp20-reservations-cmp20"
    *    role-name="persons-cmp20-has-reservations-cmp20"
    *
    * @jboss.relation-mapping style="foreign-key"
    *
    *
     * @ejb.value-object
    *   aggregate="airline.interfaces.ReservationLightValue"
    *   aggregate-name="Reservation"
    *   members="airline.interfaces.ReservationLocal"
    *   members-name="Reservation"
    *   match="all"
    *   relation="external"
    *   type="java.util.Collection"
    */
   public abstract java.util.Collection getReservations();

   /**
    * Sets a collection of related airline.interfaces.ReservationLocal
    *
    * @param reservations a collection of related airline.interfaces.ReservationLocal
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setReservations(java.util.Collection reservations);

   /**
    * This is a bi-directional many-to-many relationship CMR method
    *
    * @return a airline.interfaces.FlightLocal of related airline.interfaces.FlightLocal.
    *
    * @ejb.interface-method view-type="local"
    *
    * @ejb.relation
    *    name="flights-cmp20-persons-cmp20"
    *    role-name="persons-cmp20-has-flights-cmp20"
    *
    * @weblogic.relation join-table-name="reservations"
    *
    * @jboss.relation-mapping style="relation-table"
    * @jboss.relation-table table-name="reservations"
    *
    *
    * @weblogic.column-map
    *    foreign-key-column="person_id_fk"
    *    key-column="person_id"
    *
    * @jboss.target-relation
    *    fk-constraint="true"
    *    fk-column="person_id_fk"
    *    related-pk-field="personId"
    *
      * @weblogic.target-column-map
    *    foreign-key-column="flight_id_fk"
    *    key-column="flight_id"
    *
    * @jboss.relation
    *    fk-constraint="true"
    *    fk-column="flight_id_fk"
    *    related-pk-field="flightId"
    *
    * @ejb.value-object
    *   aggregate="airline.interfaces.FlightLightValue"
    *   aggregate-name="Flight"
    *   members="airline.interfaces.FlightLocal"
    *   members-name="Flight"
    *   match="all"
    *   relation="external"
    *   type="java.util.Collection"
    */
   public abstract java.util.Collection getFlights();

   /**
    * Sets a collection of related airline.interfaces.FlightLocal
    *
    * @param flights a collection of related airline.interfaces.FlightLocal
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setFlights(java.util.Collection flights);

   /**
    * This create method takes only mandatory (non-nullable) parameters.
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param firstName the firstName value
    * @param lastName the lastName value
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public java.lang.Integer ejbCreate( java.lang.String firstName, java.lang.String lastName ) throws javax.ejb.CreateException {
      // Use Middlegen's Sequence Block PK generator. Only works for numeric fields
      try {
         airline.interfaces.SequenceSessionLocal sequenceGenerator = 
         airline.interfaces.SequenceSessionUtil.getLocalHome().create();      
         setPersonId(new java.lang.Integer(Integer.toString(sequenceGenerator.getNextSequenceNumber( "persons.person_id" ))));
      } catch( javax.naming.NamingException e ) {
         throw new javax.ejb.CreateException(e.getMessage());
      }
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      // Set CMP fields
      setFirstName(firstName);
      setLastName(lastName);
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes this method immediately after it calls ejbCreate. 
    *
    * @param firstName the firstName value
    * @param lastName the lastName value
    */
   public void ejbPostCreate( java.lang.String firstName, java.lang.String lastName ) throws javax.ejb.CreateException {
      // Set CMR fields
   }

  
   /**
    * Method required by new version of xdoclet? Might be a bug, this seems to be the quickest fix.
    */
   public void makeDirty() {}

   /**
    * Return the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract airline.interfaces.PersonLightValue getPersonLightValue();

   /**
    * Set the light value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setPersonLightValue(airline.interfaces.PersonLightValue value);

   /**
    * Return the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract airline.interfaces.PersonValue getPersonValue();
   
   /**
    * Set the value object version of this entity.
    *
    * @ejb.interface-method view-type="local"
    */
   public abstract void setPersonValue(airline.interfaces.PersonValue value);

   /**
    * Create and return a value object populated with the data from
    * this bean.
    *
    * Standard method that must be on all Beans for the TreeBuilder to
    * work its magic.
    *
    * @return Returns a value object containing the data within this bean.
    *
    * @ejb.interface-method view-type="local"
    */
   public airline.interfaces.PersonValue getValueObject() {
      airline.interfaces.PersonValue valueObject = new airline.interfaces.PersonValue();
      valueObject.setFirstName(getFirstName());
      valueObject.setLastName(getLastName());
      return valueObject;
   }

   /**
    * Creates an instance based on a value object
    *
    * When the client invokes a create method, the EJB container invokes the ejbCreate method. 
    * Typically, an ejbCreate method in an entity bean performs the following tasks: 
    * <UL>
    * <LI>Inserts the entity state into the database.</LI>
    * <LI>Initializes the instance variables.</LI>
    * <LI>Returns the primary key.</LI>
    * </UL>
    *
    * @param value the value object used to initialise the new instance
    * @return the primary key of the new instance
    *
    * @ejb.create-method
    */
   public java.lang.Integer ejbCreate(airline.interfaces.PersonLightValue value) throws javax.ejb.CreateException {
      // Use Middlegen's Sequence Block PK generator. Only works for numeric fields
      try {
         airline.interfaces.SequenceSessionLocal sequenceGenerator = 
         airline.interfaces.SequenceSessionUtil.getLocalHome().create();      
         setPersonId(new java.lang.Integer(Integer.toString(sequenceGenerator.getNextSequenceNumber( "persons.person_id" ))));
      } catch( javax.naming.NamingException e ) {
         throw new javax.ejb.CreateException(e.getMessage());
      }
      // Use XDoclet's GUID generator. Only works for String fields
      // This requires <utilobject includeGUID="true"/> in ejbdoclet.

      setFirstName(value.getFirstName());
      setLastName(value.getLastName());
      // EJB 2.0 spec says return null for CMP ejbCreate methods.
      return null;
   }

   /**
    * The container invokes this method immediately after it calls ejbCreate. 
    *
    * @param value the value object used to initialise the new instance
    */
   public void ejbPostCreate(airline.interfaces.PersonLightValue value) throws javax.ejb.CreateException {
   }
   // No C:\temp\middlegen-2.1\samples/src/middlegen/cmp20-persons-class-code.txt found.

}
